/*
 * Copyright (C) 2010-2012 O01eg <o01eg@yandex.ru> 
 *
 * This file is part of Genetic Function Programming.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include "vm/object.h"

/// \brief Signleton for access to config files.
class Config
{
public:
	static Config& Instance()
	{
		static Config conf;
		return conf;
	}

	signed long GetSLong(const std::string& name, signed long def) const;
private:
	Config();
	~Config();

	Config(const Config&) = delete; ///< Prevent copy-constructor.
	Config(Config&&) = delete; ///< Prevent move-constructor.
	Config& operator=(const Config&) = delete; ///< Prevent assing.

	std::unordered_map<std::string, VM::WeakObject> m_Options; // Map of options
	VM::Environment m_Env; // Environment with options
	VM::Object m_List; // Pointer to options list
};

extern const size_t MAX_FUNCTIONS;

#endif

